#include <kernel.h>
#include <sys/types.h>
#include <libgte.h>
#include <libgpu.h>
#include <libgs.h>
#include <libetc.h>
#include <libcd.h>
#include <libsnd.h>
#include <libspu.h>
//#include <libmcrd.h>

#include "pcklib.c"
#include "qlplib.c"
#include "timlib.c"
#include "xalib.c"
#include "cdmenu.c"

// Internal string stuff
#define SAVEFILE_NAME	"BAMTEK00MARI_SAVE"
#define SAVE_NAME		"Marilyn: Saved Game Data"


// Sound defines
#define NUM_SOUNDS		16

#define SND_CHOOSE		0
#define SND_SELECT		1
#define SND_RETURN		2
#define SND_DENIED		3
#define SND_JUMP		4
#define SND_HURT		5
#define SND_DEATH		6
#define SND_GETCOIN		7
#define SND_POWERUP		8
#define SND_SHOOT1		9
#define SND_FIREHIT		10
#define SND_NATDIE		11
#define SND_POWERDOWN	12
#define SND_ROCKSMASH	13
#define SND_HITWALL		14
#define SND_ONEUP		15


// Buffer size defines
#define GBLBUFF_SIZE	1024*256
#define GFXBUFF_SIZE	1024*256
#define HDRBUFF_SIZE	1024*256


// Internal defines
#define PAUSE_OFF		0
#define PAUSE_BUTPLAYER	1
#define PAUSE_ALL		2

#define CHR_CROSS		123	//	{
#define CHR_CIRCLE		124	//	|
#define CHR_SQUARE		125	//	}
#define CHR_TRIANGLE	126	//	~
#define CHR_START		127	//	⌂
#define CHR_SELECT		128	//	Ç

#define CHR_UP			129	//	ü
#define CHR_DOWN		130	//	é
#define CHR_LEFT		131	//	â
#define CHR_RIGHT		132	//	ä

#define CHR_L1			133	//	à
#define CHR_R1			134	//	å
#define CHR_L2			135	//	ç
#define CHR_R2			136	//	ê


// Aliases
#define SPRITE		GsSPRITE


// Some more includes
#include "..\structs.c"


// Ordering tables and packet buffers for the graphics system
#define OT_LENGTH	10
#define PACKETMAX	2048
#define PACKETMAX2	PACKETMAX*24

GsOT 		myOT[2];
GsOT_TAG 	myOT_TAG[2][1<<OT_LENGTH];
PACKET		GPUPacketArea[2][PACKETMAX2];
int			myActiveBuff=0;


// Video mode settings
int	ScreenXres=320;
int	ScreenYres=240;
int	ScreenCenterX=0;
int	ScreenCenterY=0;
int DisplayXpos=0;
int DisplayYpos=0;
int CenterPosX=0;
int	AspectMode=0;
int VideoMode=0;


// For transitioning effects
int TransState=0;
int TransCount=0;
int TransOnGoing=0;


typedef struct {
	u_char MusicVol;
	u_char SfxVol;
	u_char SoundMode;
	u_char AspectMode;
	u_char VideoMode;
	u_char pad[123];
} CONFIG;


typedef struct {
	u_char	red;
	u_char	grn;
	u_char	blu;
	u_char	pad;
} CLRCOL;

typedef struct {
	u_short x;
	u_short y;
	u_short index;
	u_short tag;
} SPRITEOBJ;

typedef struct {
	u_short x1,y1;
	u_short x2,y2;
	u_short	hitag;
	u_short lotag;
} SECTOR;

typedef struct {
	u_char 	mode;
	u_char 	range;
	u_char 	rate;
	u_char 	frame;
} TILEANM;

typedef struct {
	
	u_char	ID[3];
	u_char	Version;
	long	BlockSize;
	
	u_char	TrackNum;
	u_char  dummy;
	struct {
		u_char NoFairy:1;
		u_char dummy:7;
		u_char dummy2:8;
	} Flags;
	
	short	xdim;
	short	ydim;
	short	NumSprites;
	short	NumSectors;
	
	u_long	SpriteEntry_ptr;
	u_long	SectorEntry_ptr;
	u_long	TxtData_ptr;
	u_long	FGtiles_ptr;
	u_long	BGtiles_ptr;
	
	CLRCOL	ClearColor;
	
	short	BGdistRate;
	short	BGreserved;
	
	u_char	BGactive[4];
	struct {
		u_char	xwrap:1;
		u_char	ywrap:1;
		u_char	xscroll:3;
		u_char	yscroll:3;
	} BGscrollMode[4];
	
	u_short	BGxOffset[4], BGyOffset[4];
	u_short BGtilemap[4][16][32];
	
	u_char	TileHitMode[256];
	TILEANM	TileAnim[256];
	
} LVLHEADER;

struct {
	int x,y;
	int fx,fy;
	int dx,dy;
} Camera;


// Global image and sprite entries
GsIMAGE		uiGraphTim;
GsSPRITE	uiCellSprite;

GsIMAGE		TempTim[4];

GsIMAGE		PlayerSpritesTim[2];
GsIMAGE		EntitySpritesTim;
GsIMAGE		CharacterSpritesTim[1];

GsSPRITE	CharacterSprite[1];


int			MenuPresent=0;
int			DefaultCardSlot=0;

int			PauseAll=0,FixCamera;
int			Flash=0,FlashCount=0,Bobbing=0,BobbingCount=0;
char		StringTemp[64]={0};
char		StringTemp2[64]={0};
char		CharTemp[2]={0};
CLRCOL		ClearColor;
PckTOC		PackTOC[4];

GsBG		BGparam;
float		BGscrollX[4];
float		BGscrollY[4];

int			tDrawXdim;
int			tDrawYdim;

LVLHEADER	*LvlHeader_ptr;
u_char		*LvlData_ptr;
GsSPRITE	*FGtileSprite_ptr;
GsCELL		*BGtileCell_ptr;
SECTOR		*SectorList_ptr;


// Function prototypes
int		GameLoop();

void	FixStuff();
void	DoWavyEffect();
void	GetInput();

void	LoadLevelHeader(PckTOC *toc, char *filename);
int		PlayLevel(PckTOC *toc, char *filename);


void	DeleteSector(int num);

void	AddCharacter(short x, short y, u_short Type, u_short Tag, short Health);
void	DeleteCharacter(int num);
void	DoCharacters();

void	AddEntity(short x, short y, int xMove, int yMove, u_short Type, u_short Flag);
void	DeleteEntity(int EntNum);
void	DoEntities();

void	AddProjectile(short x, short y, int xMove, int yMove, u_short Type, u_short Owner);
void	DeleteProjectile(int ProjNum);
void	DoProjectiles();

void	AddEffect(short x, short y, int xMove, int yMove, u_short Type, GsSPRITE *Sprite, u_char r, u_char g, u_char b);
void	DeleteEffect(int num);
void	DoEffects();

void	TransitionOut();
void	TransitionIn();
void	DoTransitions();
void	DoWinSequence(int mode, int count);

void	DrawPlayerSprite(PLAYERSTUFF *pstuff);
void	DrawFairySprite(PLAYERSTUFF *pstuff);

void	DrawUiCell(short x, short y, short cx, short cy, short cw, short ch, char trans, char r, char g, char b);
void	DrawMenuPanel(int x1, int y1, int x2, int y2, int DrawTop);
void	DrawText(char *printtext, short x, short y, char trans, char r, char g, char b);
void	DrawTextSmall(char *text, short x, short y, char trans, char r, char g, char b);
void	DrawHScrollBar(int x, int y, int length, int val, int maxval);
void	DrawVScrollBar(int x, int y, int length, int val, int maxval);

void	LoadGlobalStuff();
void	LoadTitleGraphics();
void	LoadGameGraphics();

void	DrawFG();
void	DrawBG();

void	init();

void	CheckCards();

void	PrepDisplay();
void	Display();
void	Set4x3();
void	Set16x9();
void	SetNTSC();
void	SetPAL();

int		InSector(int x, int y);
int		SpriteVisible(GsSPRITE sprite);
void	SortImage(int x, int y, GsIMAGE TimImage, u_char red, u_char grn, u_char blu);

int		CollideR(int x, int y, int mx, int my, int *cx);
int		CollideL(int x, int y, int mx, int my, int *cx);
int		CollideT(int x, int y, int mx, int my, int *cy);
int		CollideB(int x, int y, int mx, int my, int *cy);
int		GetCollideMode(int x, int y);

float	frand();
int		SpriteVisible(GsSPRITE sprite);
int		Inbox(int x, int y, int x1, int y1, int x2, int y2);
int		q2sector(int size);
int		wrap(int value, int range);
void	ErrorMessage(char *message);


// Sound system stuff
u_char	ss_cdvol	=125;
u_char	ss_sfxvol	=135;
u_char	ss_stereo	=1;

int		ss_nextvoice=0;
char	ss_malloc_table[SPU_MALLOC_RECSIZ*(NUM_SOUNDS+1)];
long	ss_vag_ptr[NUM_SOUNDS];

void	LoadVAG(int slot, u_char *vagfile_addr, int vagfile_size);
void	PlaySound(int slot, int voice, int pitch, int pan);


typedef struct {
	char*	FileName; 
	int		StartPos;
	int		EndPos;
} XAFILESTRUCT;

XAFILE XAfiles[] = {
	"\\MARILYN\\STREAMS\\MUSIC00.XA;1", 0,
};